@@ -7,12 +7,24 @@ use Tickit::Async;
 use Tickit::Widgets qw( SegmentDisplay HBox Border );
 use IO::Async::Loop;
 use IO::Async::Timer::Absolute;
+
+use Getopt::Long;
+use List::Util qw( any );
 use POSIX qw( strftime );
 
+GetOptions(
+   'style|S=s' => \( my $STYLE = "halfline" ),
+) or exit 1;
+
+any { $STYLE eq $_ } qw( halfline unicode linedraw ) or
+   die "Expected --style to be halfline|unicode|linedraw\n";
+
+my $THICKNESS = ( $STYLE eq "halfline" ) ? 1 : 2;
+
 my $loop = IO::Async::Loop->new;
 
-my @digits = map { Tickit::Widget::SegmentDisplay->new( type => '7', use_halfline => 1, thickness => 1 ) } 0 .. 5;
-my @colons = map { Tickit::Widget::SegmentDisplay->new( type => ':', use_halfline => 1, thickness => 1 ) } 0 .. 1;
+my @digits = map { Tickit::Widget::SegmentDisplay->new( type => '7', "use_$STYLE" => 1, thickness => $THICKNESS ) } 0 .. 5;
+my @colons = map { Tickit::Widget::SegmentDisplay->new( type => ':', "use_$STYLE" => 1, thickness => $THICKNESS ) } 0 .. 1;
 
 my $nexttime = time;
 $loop->add( IO::Async::Timer::Absolute->new(
